const DATABASE: &str = "server_tracker";

use configparser::ini::Ini;
use influxdb::{Client, InfluxDbWriteable, ReadQuery, Timestamp};
use std::str::FromStr;

use crate::ServerResponse;

pub async fn connect(config: &Ini) -> Result<Client, eyre::Report> {
    let addr: String = config
        .get("influx", "socketaddr")
        .unwrap_or_else(|| "127.0.0.1:8086".into());
    let https: bool = config
        .get("influx", "https")
        .map(|f| bool::from_str(&f).unwrap_or(false))
        .unwrap_or(false);
    let mut host = String::from("http");
    if https {
        host.push('s');
    }
    host.push_str("://");
    host.push_str(&addr);

    let client;
    if config.get("influx", "username").is_some() {
        let username: String = config
            .get("influx", "username")
            .unwrap_or_else(|| "127.0.0.1:8086".into());
        let password: String = config
            .get("influx", "password")
            .unwrap_or_else(|| "127.0.0.1:8086".into());
        client = Client::new(host, DATABASE).with_auth(username, password);
    } else {
        client = Client::new(host, DATABASE);
    }
    // Standard behavior of Influx is to always create a new DB, and a failure-
    // to do so means the connection doesn't work.
    let query = format!("CREATE DATABASE {}", DATABASE);
    client.query(ReadQuery::new(query)).await?;
    Ok(client)
}

pub async fn write_playercount(
    client: &Client,
    server: &ServerResponse<'_>,
) -> Result<String, eyre::Report> {
    let influx_query = Timestamp::from(server.time)
        .into_query("playercount")
        .add_field("clients", server.clients)
        .add_tag("server_name", server.server.name.clone())
        .add_tag("map", server.map.clone().unwrap_or_else(|| "NONE".into()));
    Ok(client.query(influx_query).await?)
}
